# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 

from Products.CMFCore.utils import format_stx, getToolByName
from DocumentTemplate.DT_Util import html_quote

import traceback
from striphtml import strip

def forum_parser(self, content, emoticons=1, add_breaks=1,
                 preview=None, strip_html=0, text_format='text/plain',
                 parsing=1):

    if strip_html:
        tmp_content = strip(self, content)
    else:
        tmp_content = content

    if text_format in ('plain', 'text/plain'):
        tmp_content = html_quote(content)
    elif text_format in ('structured-text', 'text/structured-text'):
        tmp_content = format_stx(text=content)

    if parsing:
        # Miscellaneous html tags (presentation)
        tmp_content = tmp_content.replace('[b]','<b>')
        tmp_content = tmp_content.replace('[/b]','</b>')
        tmp_content = tmp_content.replace('[i]','<i>')
        tmp_content = tmp_content.replace('[/i]','</i>')
        tmp_content = tmp_content.replace('[big]','<span style="font-size:large;">')
        tmp_content = tmp_content.replace('[/big]','</span>')
        tmp_content = tmp_content.replace('[img]', '<img src="')
        tmp_content = tmp_content.replace('[/img]', '" border="0" />')
        tmp_content = tmp_content.replace('[blockquote]', '<blockquote>')
        tmp_content = tmp_content.replace('[/blockquote]', '</blockquote>')
        tmp_content = tmp_content.replace('[ol]', '<ol>')
        tmp_content = tmp_content.replace('[ul]', '<ul>')
        tmp_content = tmp_content.replace('[/ol]', '</ol>')
        tmp_content = tmp_content.replace('[/ul]', '</ul>')
        tmp_content = tmp_content.replace('[li]', '<li>')
        tmp_content = tmp_content.replace('[/li]', '</li>')
        # Unsupported/Nonstandard M$-style
        tmp_content = tmp_content.replace('[blink]','<blink>')
        tmp_content = tmp_content.replace('[/blink]','</blink>')
        tmp_content = tmp_content.replace('[u]','<u>')
        tmp_content = tmp_content.replace('[/u]','</u>')
        tmp_content = tmp_content.replace('[marquee]','<marquee>')
        tmp_content = tmp_content.replace('[/marquee]','</marquee>')

        # <a href tags..
        # Only convert if the url is well-formed, ignore otherwise
        url_check1 = '[url='
        url_check2 = ' urltitle='
        url_check3 = '][/url]'
        if tmp_content.find(url_check1) >= 0 \
               and tmp_content.find(url_check2) >= 0 \
               and tmp_content.find(url_check3) >= 0:
            tmp_content = tmp_content.replace(
                '[url=', '<a target="_blank" title="External Link" href="')
            tmp_content = tmp_content.replace(' urltitle=', '">')
            tmp_content = tmp_content.replace('][/url]', '</a>')
        
        # Code tags, I am not converting straight [code] to <code>
        # since I need the paragraph break that <pre> gives, suit this
        # to your needs
        tmp_content = tmp_content.replace('[code]', '<pre>')
        tmp_content = tmp_content.replace('[/code]', '</pre>')


        portal = getToolByName(self, 'portal_url').getPortalObject()
        if hasattr(portal, 'fcEmoticons'):
            img_path = portal.fcEmoticons.absolute_url()
            if emoticons:
                tmp_content = tmp_content.replace(':cool:', '<img alt="cool!" src="%s/cool.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(':mad:', '<img alt="mad" src="%s/mad.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(':grin:', '<img alt="grin" src="%s/grin.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(':huh:', '<img alt="huh?" src="%s/huh.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(':laugh:', '<img alt="laugh!" src="%s/laugh.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(':sad:', '<img alt="sad" src="%s/sad.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(':(', '<img alt="sad" src="%s/sad.gif" width="20" height="20" border="0" />' % img_path) 
                tmp_content = tmp_content.replace(':smile:', '<img alt="smile!" src="%s/smile.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(':)', '<img alt="smile" src="%s/smile.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(':wacko:', '<img alt="wacko" src="%s/wacko.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(':wink:', '<img alt="wink" src="%s/wink.gif" width="20" height="20" border="0" />' % img_path)
                tmp_content = tmp_content.replace(';)', '<img alt="wink" src="%s/wink.gif" width="20" height="20" border="0" />' % img_path)


    if text_format in ('plain','text/plain') and int(add_breaks):
        tmp_content = tmp_content.replace('\n', '<br />')

    # if preview page calls this, then I need to re-strip all the "safe"
    # html converted, otherwise, the method might return a malformed html
    # unterminated tag due to the "preview" status (i.e. called from the
    # main topic display page, which should only show a specific number
    # of characters, The "preview" flag is used on the main topic page,
    # where only partial message contents are shown..:
    if strip_html:
        tmp_content = strip(self, tmp_content)

    if preview is not None:
        return strip(self, tmp_content)
    else:
        return tmp_content


# word-wrap function that preserves line breaks and spaces
# (from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/148061)
def wrap(s, width=40):
    return reduce(lambda line, word, width=width: '%s%s%s' %
                  (line,
                   ' \n'[(len(line[line.rfind('\n')+1:])
                          + len(word.split('\n',1)[0]
                                ) >= width)],
                   word),
                  s.split(' ')
                  )
